@@ -6,7 +6,7 @@ use Mojo::JSON;
 use Mojo::Util 'deprecated';
 use constant DEBUG => $ENV{MOJO_OPENAPI_DEBUG} || 0;
 
-our $VERSION = '1.10';
+our $VERSION = '1.11';
 
 sub EXCEPTION {
   +{json => {errors => [{message => 'Internal server error.', path => '/'}]}, status => 500};
@@ -74,7 +74,7 @@ sub _add_routes {
     $route_prefix = "$spec_route_name.";
   }
 
-  for my $path (sort { length $a <=> length $b } keys %$paths) {
+  for my $path (_sort_paths(keys %$paths)) {
     next if $path =~ $X_RE;
     my @parameters = @{$paths->{$path}{parameters} || []};
     my $route_path = $path;
@@ -107,7 +107,7 @@ sub _add_routes {
 
       $endpoint->to(ref $to eq 'ARRAY' ? @$to : $to) if $to;
       $endpoint->to({'openapi.op_spec' => $op_spec});
-      warn "[OpenAPI] Add route $http_method @{[$endpoint->render]}\n" if DEBUG;
+      warn "[OpenAPI] Add route $http_method $path (@{[$endpoint->render]})\n" if DEBUG;
     }
 
     unless ($has_options) {
@@ -171,7 +171,17 @@ sub _reply {
 
 sub _render {
   my ($renderer, $c, $output, $options) = @_;
-  exists $c->stash->{openapi} or return;
+
+  # fallback to default renderer
+  unless (exists $c->stash->{openapi}) {
+    my $renderer = $c->app->renderer;
+    my $handler  = $renderer->handlers->{$renderer->default_handler};
+    $c->app->log->debug(
+      "Using default_handler to render data since 'openapi' was not found in stash. Set 'handler' in stash to avoid this message."
+    );
+    local $options->{handler} = $renderer->default_handler;
+    return $renderer->$handler($c, $output, $options);
+  }
 
   my $self = $c->stash('openapi.object') or return;
   my $status = $c->stash('status') || 200;
@@ -191,8 +201,10 @@ sub _render {
 }
 
 sub _render_json {
-  $_[0]->res->headers->content_type('application/json;charset=UTF-8');
-  return Mojo::JSON::encode_json($_[1]);
+  my $c = shift;
+  return $_[0]->slurp if UNIVERSAL::isa($_[0], 'Mojo::Asset');
+  $c->res->headers->content_type('application/json;charset=UTF-8');
+  return Mojo::JSON::encode_json($_[0]);
 }
 
 sub _render_route_spec {
@@ -240,6 +252,13 @@ sub _serialize {
   Data::Dumper->new([@_])->Indent(1)->Pair(': ')->Sortkeys(1)->Terse(1)->Useqq(1)->Dump;
 }
 
+sub _sort_paths {
+  return
+    map { $_->[0] }
+    sort { $a->[1] <=> $b->[1] || length $a->[0] <=> length $b->[0] }
+    map { [$_, $_ =~ /\{/ ? 1 : 0] } @_;
+}
+
 sub _validate {
   my ($c, $args) = @_;
   my $self    = $c->stash('openapi.object');
